/* **********************************************************
 * Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
 * **********************************************************/

#include "driver-config.h"
#include <linux/string.h>       // for memset

#include "vmware.h"
#include "machine.h"
#include "modulecall.h"
#include "driver_vmcore.h"

/*
 *----------------------------------------------------------------------
 *
 * LinuxDriver_CheckPadding --
 *
 *      Check for expected padding.
 *      This check currently fails on the egcs compiler.
 *      Also check size of crosspage and offset of linearaddress.
 *
 * Results:
 *      TRUE if the check succeeds -- module will be loaded
 *
 * Side effects:
 *      Output to kernel log on error.
 *
 *----------------------------------------------------------------------
 */

Bool
LinuxDriver_CheckPadding(void)
{
   DTRWords dtr;
   uint16 *x;

   memset(&dtr, 0, sizeof dtr);
   dtr.dtr.limit = 0x1111;
   dtr.dtr.offset = 0x22223333;

   x = (uint16*)&dtr;

   if (x[0] == 0x1111 && x[1] == 0x3333 && x[2] == 0x2222) {
   } else {
      Warning("DTR padding\n");
      goto error;
   }

   return TRUE;

error:
   return FALSE;
}
